==> 0.2.9.tar.gz: lib/chobi_admob_banner_ad.dart <==

import 'dart:io';

import 'package:chobi_admob_flutter/chobi_admob.dart';
import 'package:chobi_admob_flutter/chobi_admob_flutter_platform_interface.dart';
import 'package:flutter/cupertino.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';


// TODO: create the class holds banner ad status -> available, loaded, ad object, and so on.
// TODO: admob initialize after UMP CHECK


class BannerState {

  final bool isBannerAvailable;
  final bool isBannerAdLoaded;

  final BannerAd? bannerAd;

  final bool appTrackingConfirmed;

  BannerState({
    this.isBannerAvailable = false,
    this.isBannerAdLoaded = false,
    this.bannerAd,
    this.appTrackingConfirmed = false,
  });
}

class ChobiAdMobBannerAd extends ConsumerStatefulWidget {

  final String? androidBannerId;
  final String? iosBannerId;
  final String lastClickedPrefsKey;
  final List<String>? testDeviceIds;
  final bool isDebug;
  final int adIntervalTimeMs;

  final Function(ConsentStatus status)? additionalConsentCheckFinished;
  final Function(FormError error)? additionalConsentError;

  final BannerAdListener? bannerAdListener;


  final StateProvider<BannerState>? bannerStateProvider;

  final bool needToConfirmIosAppTracking;

  final Widget Function(BuildContext context, BannerAd? ad) builder;

  const ChobiAdMobBannerAd({
    super.key,
    this.androidBannerId,
    this.iosBannerId,
    required this.lastClickedPrefsKey,
    this.testDeviceIds,
    this.isDebug = true,
    this.adIntervalTimeMs = 5 * 60 * 1000,
    this.additionalConsentCheckFinished,
    this.additionalConsentError,
    this.bannerAdListener,
    this.bannerStateProvider,
    this.needToConfirmIosAppTracking = true,
    required this.builder,
  });


  @override
  ConsumerState<ChobiAdMobBannerAd> createState() => ChobiAdMobBannerAdState();
}


const _blankBox = SizedBox.shrink(key: ValueKey('chobi_admob_banner_ad_blank_box'),);

class ChobiAdMobBannerAdState extends ConsumerState<ChobiAdMobBannerAd> with AutomaticKeepAliveClientMixin {
//class ChobiAdMobBannerAdState extends ConsumerState<ChobiAdMobBannerAd> {

  @override
  bool get wantKeepAlive => true;

  late ChobiAdMob? _chobiAdMob;

  late final StateProvider<BannerState> _bannerStateProvider;

  void _updateBannerState(WidgetRef ref, {
    bool? isBannerAvailable,
    bool? isBannerAdLoaded,
    BannerAd? Function()? bannerAdGetter,
    bool? appTrackingConfirmed,
  }) {
    final oldState = ref.read(_bannerStateProvider);
    ref.read(_bannerStateProvider.notifier).state = BannerState(
      isBannerAvailable: isBannerAvailable ?? oldState.isBannerAvailable,
      isBannerAdLoaded: isBannerAdLoaded ?? oldState.isBannerAdLoaded,
      bannerAd: (bannerAdGetter != null) ? bannerAdGetter() : oldState.bannerAd,
      appTrackingConfirmed: appTrackingConfirmed ?? oldState.appTrackingConfirmed,
    );
  }



  @override
  void initState() {
    super.initState();

    _bannerStateProvider = widget.bannerStateProvider ?? StateProvider((ref) => BannerState());

    WidgetsBinding.instance.addPostFrameCallback((_) async {
      if (Platform.isIOS && widget.needToConfirmIosAppTracking) {
        await ChobiAdmobFlutterPlatform.instance.confirmAppTrackingTransparency();
      }

      _updateBannerState(ref, appTrackingConfirmed: true);

      await ChobiAdMob.loadLastClickedTimeFromPrefs(prefKey: widget.lastClickedPrefsKey).then((value) {
        _chobiAdMob = ChobiAdMob(
          adLastClickedTime: value,
          lastClickedPrefsKey: widget.lastClickedPrefsKey,
          isDebug: widget.isDebug,
          testDeviceIds: widget.testDeviceIds,
          androidId: widget.androidBannerId,
          iosId: widget.iosBannerId,
          androidTestId: ChobiAdMob.androidTestAdaptiveBannerId,
          iosTestId: ChobiAdMob.iosTestAdaptiveBannerId,
          adIntervalTimeMs: widget.adIntervalTimeMs,
        )..onConsentCheckFinished =  (status) {
          _onConsentCheckFinished(status);
          widget.additionalConsentCheckFinished?.call(status);
        }..onConsentCheckError = (err) {
          _onConsentError(err);
          widget.additionalConsentError?.call(err);
        }..checkPermissionAndStartAdProcess();
      });
    });
  }


  Future<void> _innerBannerStopProcess() async {
    await _chobiAdMob?.stopAdProcess();
    await disposeBanner();
  }

  @override
  void dispose() async {
    await _innerBannerStopProcess();

    super.dispose();
  }

  @protected
  Future<void> loadBanner() async {

    final bannerState = ref.read(_bannerStateProvider);

    if (!bannerState.isBannerAvailable) {
      debugPrint('AD not available');
      return;
    }

    if (bannerState.bannerAd != null) {
      await disposeBanner();
    }

    final bannerId = _chobiAdMob?.getAdId();
    debugPrint('banner id = $bannerId');

    if (bannerId == null) {
      debugPrint('banner id is null');
      return;
    }

    final cWidth = MediaQuery.of(context).size.width.truncate();
    final adSize = await AdSize.getCurrentOrientationAnchoredAdaptiveBannerAdSize(cWidth);


    if (adSize == null) {
      debugPrint('adSize is null');
      return;
    }

    final ad = BannerAd(
      size: adSize,
      adUnitId: bannerId,
      listener: generateBannerListener(),
      request: const AdRequest(),
    );

    //_updateBannerState(ref, bannerAdGetter: () => ad);

    await ad.load();
  }

  @protected
  Future<void> disposeBanner() async {
    ref.read(_bannerStateProvider).bannerAd?.dispose();
    ref.read(_bannerStateProvider.notifier).state = BannerState();
  }

  @protected
  BannerAdListener generateBannerListener() {
    return BannerAdListener(
        onAdClosed: widget.bannerAdListener?.onAdClosed,
        onAdImpression: widget.bannerAdListener?.onAdImpression,
        onAdOpened: widget.bannerAdListener?.onAdOpened,
        onAdWillDismissScreen: widget.bannerAdListener?.onAdWillDismissScreen,
        onPaidEvent: widget.bannerAdListener?.onPaidEvent,

        onAdLoaded: (ad) {
          _updateBannerState(ref, isBannerAdLoaded: true, bannerAdGetter: () => ad as BannerAd);
          widget.bannerAdListener?.onAdLoaded?.call(ad);
        },
        onAdFailedToLoad: (ad, err) async {
          debugPrint('admob banner laod error: ${err.message}');
          _updateBannerState(ref, isBannerAdLoaded: false);
          widget.bannerAdListener?.onAdFailedToLoad?.call(ad, err);
          await disposeBanner();
        },
        onAdClicked: (ad) async {
          await _chobiAdMob?.updateLastClickedTime();
          await disposeBanner();
          await _chobiAdMob?.checkPermissionAndStartAdProcess();
          widget.bannerAdListener?.onAdClicked?.call(ad);
        }
    );
  }


  void _onConsentCheckFinished(ConsentStatus status) {
    if (status != ConsentStatus.required) {
      _updateBannerState(ref, isBannerAvailable: true);
      loadBanner();
    }
  }

  void _onConsentError(FormError error) {
    _updateBannerState(ref, isBannerAvailable: false);
    debugPrint('UMP FORM ERROR[${error.errorCode}]: ${error.message}');
  }


  @override
  Widget build(BuildContext context) {
    super.build(context);

    return Consumer(
      builder: (atCtx, atRef, atChild) {
        final isAppTrackingChecked = atRef.watch(_bannerStateProvider.select((value) => value.appTrackingConfirmed));
        
        return isAppTrackingChecked ? Consumer(
          builder: (adCtx, adRef, adChild) {
            final ad = adRef.watch(_bannerStateProvider.select((value) => value.bannerAd));
            return widget.builder(adCtx, ad);
          },
        ) : _blankBox;
      },
    );
  }
}

==> 0.2.11.tar.gz: lib/chobi_admob_banner_ad.dart <==

import 'dart:io';

import 'package:chobi_admob_flutter/chobi_admob.dart';
import 'package:chobi_admob_flutter/chobi_admob_flutter_platform_interface.dart';
import 'package:flutter/cupertino.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';


// TODO: create the class holds banner ad status -> available, loaded, ad object, and so on.
// TODO: admob initialize after UMP CHECK


class BannerState {

  final bool isBannerAvailable;
  final bool isBannerAdLoaded;

  final BannerAd? bannerAd;

  final bool appTrackingConfirmed;

  BannerState({
    this.isBannerAvailable = false,
    this.isBannerAdLoaded = false,
    this.bannerAd,
    this.appTrackingConfirmed = false,
  });
}

class ChobiAdMobBannerAd extends ConsumerStatefulWidget {

  final String? androidBannerId;
  final String? iosBannerId;
  final String lastClickedPrefsKey;
  final List<String>? testDeviceIds;
  final bool isDebug;
  final int adIntervalTimeMs;

  final Function(ConsentStatus status)? additionalConsentCheckFinished;
  final Function(FormError error)? additionalConsentError;

  final BannerAdListener? bannerAdListener;


  final StateProvider<BannerState>? bannerStateProvider;

  final bool needToConfirmIosAppTracking;

  final Widget Function(BuildContext context, BannerAd? ad) builder;

  const ChobiAdMobBannerAd({
    super.key,
    this.androidBannerId,
    this.iosBannerId,
    required this.lastClickedPrefsKey,
    this.testDeviceIds,
    this.isDebug = true,
    this.adIntervalTimeMs = 5 * 60 * 1000,
    this.additionalConsentCheckFinished,
    this.additionalConsentError,
    this.bannerAdListener,
    this.bannerStateProvider,
    this.needToConfirmIosAppTracking = true,
    required this.builder,
  });


  @override
  ConsumerState<ChobiAdMobBannerAd> createState() => ChobiAdMobBannerAdState();
}


const _blankBox = SizedBox.shrink(key: ValueKey('chobi_admob_banner_ad_blank_box'),);

class ChobiAdMobBannerAdState extends ConsumerState<ChobiAdMobBannerAd> with AutomaticKeepAliveClientMixin {
//class ChobiAdMobBannerAdState extends ConsumerState<ChobiAdMobBannerAd> {

  @override
  bool get wantKeepAlive => true;

  late ChobiAdMob? _chobiAdMob;

  late final StateProvider<BannerState> _bannerStateProvider;

  void _updateBannerState(WidgetRef ref, {
    bool? isBannerAvailable,
    bool? isBannerAdLoaded,
    BannerAd? Function()? bannerAdGetter,
    bool? appTrackingConfirmed,
  }) {
    final oldState = ref.read(_bannerStateProvider);
    ref.read(_bannerStateProvider.notifier).state = BannerState(
      isBannerAvailable: isBannerAvailable ?? oldState.isBannerAvailable,
      isBannerAdLoaded: isBannerAdLoaded ?? oldState.isBannerAdLoaded,
      bannerAd: (bannerAdGetter != null) ? bannerAdGetter() : oldState.bannerAd,
      appTrackingConfirmed: appTrackingConfirmed ?? oldState.appTrackingConfirmed,
    );
  }



  @override
  void initState() {
    super.initState();

    _bannerStateProvider = widget.bannerStateProvider ?? StateProvider((ref) => BannerState());

    WidgetsBinding.instance.addPostFrameCallback((_) async {
      if (Platform.isIOS && widget.needToConfirmIosAppTracking) {
        await ChobiAdmobFlutterPlatform.instance.confirmAppTrackingTransparency();
      }

      _updateBannerState(ref, appTrackingConfirmed: true);

      await ChobiAdMob.loadLastClickedTimeFromPrefs(prefKey: widget.lastClickedPrefsKey).then((value) {
        _chobiAdMob = ChobiAdMob(
          adLastClickedTime: value,
          lastClickedPrefsKey: widget.lastClickedPrefsKey,
          isDebug: widget.isDebug,
          testDeviceIds: widget.testDeviceIds,
          androidId: widget.androidBannerId,
          iosId: widget.iosBannerId,
          androidTestId: ChobiAdMob.androidTestAdaptiveBannerId,
          iosTestId: ChobiAdMob.iosTestAdaptiveBannerId,
          adIntervalTimeMs: widget.adIntervalTimeMs,
        )..onConsentCheckFinished =  (status) async {
          await _onConsentCheckFinished(status);
          widget.additionalConsentCheckFinished?.call(status);
        }..onConsentCheckError = (err) {
          _onConsentError(err);
          widget.additionalConsentError?.call(err);
        }..checkPermissionAndStartAdProcess();
      });
    });
  }


  Future<void> _innerBannerStopProcess() async {
    await _chobiAdMob?.stopAdProcess();
    await disposeBanner();
  }

  @override
  void dispose() async {
    await _innerBannerStopProcess();

    super.dispose();
  }


  bool _isAdMobInitialized = false;
  Future<void> _initAdMob() async {
    if (!_isAdMobInitialized) {
      _isAdMobInitialized = true;
      await MobileAds.instance.initialize();
    }
  }


  @protected
  Future<void> loadBanner() async {

    final bannerState = ref.read(_bannerStateProvider);

    if (!bannerState.isBannerAvailable) {
      debugPrint('AD not available');
      return;
    }

    if (bannerState.bannerAd != null) {
      await disposeBanner();
    }

    final bannerId = _chobiAdMob?.getAdId();
    debugPrint('banner id = $bannerId');

    if (bannerId == null) {
      debugPrint('banner id is null');
      return;
    }

    final cWidth = MediaQuery.of(context).size.width.truncate();
    final adSize = await AdSize.getCurrentOrientationAnchoredAdaptiveBannerAdSize(cWidth);


    if (adSize == null) {
      debugPrint('adSize is null');
      return;
    }

    final ad = BannerAd(
      size: adSize,
      adUnitId: bannerId,
      listener: generateBannerListener(),
      request: const AdRequest(),
    );

    //_updateBannerState(ref, bannerAdGetter: () => ad);

    await ad.load();
  }

  @protected
  Future<void> disposeBanner() async {
    ref.read(_bannerStateProvider).bannerAd?.dispose();
    ref.read(_bannerStateProvider.notifier).state = BannerState();
  }

  @protected
  BannerAdListener generateBannerListener() {
    return BannerAdListener(
        onAdClosed: widget.bannerAdListener?.onAdClosed,
        onAdImpression: widget.bannerAdListener?.onAdImpression,
        onAdOpened: widget.bannerAdListener?.onAdOpened,
        onAdWillDismissScreen: widget.bannerAdListener?.onAdWillDismissScreen,
        onPaidEvent: widget.bannerAdListener?.onPaidEvent,

        onAdLoaded: (ad) {
          _updateBannerState(ref, isBannerAdLoaded: true, bannerAdGetter: () => ad as BannerAd);
          widget.bannerAdListener?.onAdLoaded?.call(ad);
        },
        onAdFailedToLoad: (ad, err) async {
          debugPrint('admob banner laod error: ${err.message}');
          _updateBannerState(ref, isBannerAdLoaded: false);
          widget.bannerAdListener?.onAdFailedToLoad?.call(ad, err);
          await disposeBanner();
        },
        onAdClicked: (ad) async {
          await _chobiAdMob?.updateLastClickedTime();
          await disposeBanner();
          await _chobiAdMob?.checkPermissionAndStartAdProcess();
          widget.bannerAdListener?.onAdClicked?.call(ad);
        }
    );
  }


  Future<void> _onConsentCheckFinished(ConsentStatus status) async {
    if (status != ConsentStatus.required) {
      //await _initAdMob();
      _initAdMob();
      _updateBannerState(ref, isBannerAvailable: true);
      await loadBanner();
    }
  }

  void _onConsentError(FormError error) {
    _updateBannerState(ref, isBannerAvailable: false);
    debugPrint('UMP FORM ERROR[${error.errorCode}]: ${error.message}');
  }


  @override
  Widget build(BuildContext context) {
    super.build(context);

    return Consumer(
      builder: (atCtx, atRef, atChild) {
        final isAppTrackingChecked = atRef.watch(_bannerStateProvider.select((value) => value.appTrackingConfirmed));
        
        return isAppTrackingChecked ? Consumer(
          builder: (adCtx, adRef, adChild) {
            final ad = adRef.watch(_bannerStateProvider.select((value) => value.bannerAd));
            return widget.builder(adCtx, ad);
          },
        ) : _blankBox;
      },
    );
  }
}
